/*
 * DummyLoad_CTL.h
 *
 *  Created on: Nov 8, 2021
 *      Author: Hans
 */

#ifndef DUMMYLOAD_CTL_H_
#define DUMMYLOAD_CTL_H_

#include <Arduino.h>
#include <LiquidCrystal.h>
#include <wire.h>
#include <eeprom.h>
#include <ads1219.h>
#include "Rotary.h"
#include "FanMonitor.h"

/*****************************************************************/
/* Variables & definitions										 */
/*****************************************************************/

//LCD definition
LiquidCrystal lcd(6, 7, 8, 9, 10, 11, 12);

// User Defined LCD characters
byte LcdBlock[8] = {
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B11111,
  B11111,
  B00000,
};
//FAN definitions
#define FAN_MONITOR_PIN 14
#define FAN_PWM 5
FanMonitor FanSpeed = FanMonitor(FAN_MONITOR_PIN, FAN_TYPE_BIPOLE);



//Rotary Encoder
//Encoder is wired with the common to ground and the two outputs to pins 2 and 3.
Rotary rotary = Rotary(3, 2);
volatile int encoder_Counter = 0;
const byte ENC_BTN = 4;		//Button on rotary encoder

//ADS1219 definitions
#define address 0x40
#define drdy A3
ADS1219 ads(drdy);

//NTC definitions
#define RT0 10000   // Ohm
#define B 4085      // K
#define VCC 5    //Supply voltage
#define R 10000  //R=10Kohm
float RT, VR, ln, T0, VRT, ntc_Temp;

// Acoustic Alarm
const byte BUZZER = 13;		//Buzzer on pin 13
#define BUZZER_OFF      0
#define BUZZER_WARNING  1
#define BUZZER_ALARM    2


//TimersSoftware timers
static int timer_Btn;   //Button debounce
static int timer_Adc;	//ADC poll timer
static int timer_Temp;	//Temperatur control
static int timer_Buzzer;

//Buttons
#define BTN_ENC_PUSHED  1
unsigned char status_Btn = 0;

//Menu Definition
#define MENU_NORMAL_INI		0
#define MENU_NORMAL_1		1
#define MENU_SETUP_INI		10
#define MENU_SETUP			11
#define MENU_FREQUENCY_INI  12
#define MENU_FREQUENCY		13
#define MENU_ZERO_INI		14
#define MENU_ZERO			15
#define MENU_ATT_INI		16
#define MENU_ATT			17
#define MENU_FAN_TEST_INI	18
#define MENU_FAN_TEST		19
#define MENU_RETURN			20
#define MENU_CAL_INI		30
#define MENU_CAL			31
#define MENU_CAL_SLOPE_INI	32
#define MENU_CAL_SLOPE		33
#define MENU_CAL_OFF_INI	34
#define MENU_CAL_OFF   		35
//#define MENU_THERM_PWR_INI  40
#define MENU_THERM_PWR		 40
#define MENU_THERM_PWR_START 41
#define MENU_THERM_PWR_STOP  42
#define MENU_THERM_PWR_CALC	 41
#define MENU_THERM_PWR_RST   43




unsigned char menu_Mode = MENU_NORMAL_INI;
unsigned char Status_Buttons = 0;
boolean update_LCD = false;
boolean themalPowerActive = false;
float thermalTime = 0.0;

#define MENU_SETUP_MAIN_MAX	7
static const char sm_String_1[]PROGMEM = "Thermal Power";
static const char sm_String_2[]PROGMEM = "Def. Band";
static const char sm_String_3[]PROGMEM = "Atten.";
static const char sm_String_4[]PROGMEM = "Zero-Off.";
static const char sm_String_5[]PROGMEM = "Fan Test";
static const char sm_String_6[]PROGMEM = "F-Calib.";
static const char sm_String_7[]PROGMEM = "Exit";
static const char* const TEXT_SETUP_MAIN_LIST[] PROGMEM = {sm_String_1,sm_String_2,sm_String_3,sm_String_4,sm_String_5,sm_String_6,sm_String_7};

#define MENU_FREQ_MAX	14
static const char f_String_1[]PROGMEM =  " 160m";
static const char f_String_2[]PROGMEM =  "  80m";
static const char f_String_3[]PROGMEM =  "  60m";
static const char f_String_4[]PROGMEM =  "  40m";
static const char f_String_5[]PROGMEM =  "  30m";
static const char f_String_6[]PROGMEM =  "  20m";
static const char f_String_7[]PROGMEM =  "  17m";
static const char f_String_8[]PROGMEM =  "  15m";
static const char f_String_9[]PROGMEM =  "  12m";
static const char f_String_10[]PROGMEM = "  10m";
static const char f_String_11[]PROGMEM = "   6m";
static const char f_String_12[]PROGMEM = "   2m";
static const char f_String_13[]PROGMEM = "1.25m";
static const char f_String_14[]PROGMEM = "70cm";

static const char* const TEXT_FREQ_LIST[] PROGMEM = {f_String_1,f_String_2,f_String_3,f_String_4,f_String_5,f_String_6,
													f_String_7,f_String_8,f_String_9,f_String_10,f_String_11,f_String_12,
													f_String_13,f_String_14};


unsigned int fwd_Pwr;
long adc_Val;
float pwr_DBm1;
float input_Att1 = 0.0;
long adc_offset1 = 934000;
float ad8307_Slope1 = 23.55;
int inp_Frequency;
int adc_Cal_1[MENU_FREQ_MAX];



/* Structure definition */
struct table_1d {
    uint8_t x_length;

    float *x_values;
    float *y_values;
};

//Measurent table based on deltaT of 10C from 40C to 50C
//This is depending on the individual design and heat sink of the dummy load
static float point_x[] = {10.2, 11.2, 12.5, 13.6, 15.5, 17.0, 20.0, 23.9, 29.3, 36.8, 50.2, 72.7, 182.1, 843.7};
static float point_y[] = {1600.0, 1500.0, 1400.0, 1300.0, 1200.0, 1100.0, 1000.0, 900.0, 800.0, 700.0, 600.0, 500.0, 300.0, 100.0};

static struct table_1d point_table = {
    14,      /* Number of data points */
    point_x, /* Array of x-coordinates */
    point_y  /* Array of y-coordinates */
};



//EEPROM Addresses
#define EERPOM_FREQ      0
#define EEPROM_ATT       2
#define EEPROM_SLOPE     4
#define EEPROM_ZERO      6  //4 byte long!
#define EEPROM_CAL_1	10	//28 byte long!


//Function Prototypes
void rotate(void);
void readButtons(void);
unsigned char EncModifyChar(unsigned char,unsigned char);
int EncModifyInt(int,int);
int EEPROMReadInt(int);
long EEPROMReadLong(int);
void setFanPWM(int);
void setAlarm(char);




#endif /* DUMMYLOAD_CTL_H_ */
